/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package seco.util.conf;

import java.awt.Color;
import java.util.*;
import java.util.regex.*;

/**
 * @author     Samppa Saarela
 * @created    5. joulukuuta 2002
 */
public class Constants {
	/**
	 *  Description of the Field
	 */
	private Map constants                 = new LinkedHashMap();
	/**
	 *  Description of the Field
	 */
	private final static Pattern pattern  = Pattern.compile("\\$\\{([^\\}]*)\\}");


	/**
	 *  Description of the Method
	 *
	 * @param  key  Description of the Parameter
	 * @return      Description of the Return Value
	 */
	public boolean containsKey(String key) {
		return constants.containsKey(key);
	}


	/**
	 * @param  key
	 * @return
	 */
	public String[] getArray(String key) {
		if (!containsKey(key)) {
			return null;
		}
		List list           = new LinkedList();
		StringTokenizer st  = new StringTokenizer(getString(key), getString("ARRAY_DELIM"));
		while (st.hasMoreTokens()) {
			String token  = st.nextToken();
			list.add(parseValue(token.trim()));
		}
		return (String[])list.toArray(new String[list.size()]);
	}


	/**
	 *  Gets the boolean attribute of the YOMConstants class
	 *
	 * @param  key  Description of the Parameter
	 * @return      The boolean value
	 */
	public boolean getBoolean(String key) {
		return new Boolean(getString(key)).booleanValue();
	}


	/**
	 *  Gets the color attribute of the YOMConstants class
	 *
	 * @param  key  Description of the Parameter
	 * @return      The color value
	 */
	public Color getColor(String key) {
		try {
			return getColorValue(getString(key));
		} catch (IllegalArgumentException iae) {
			System.err.println("Error with key \"" + key + "\":\t" +
					iae.getMessage());
			return null;
		}
	}


	/**
	 *  Gets the colorValue attribute of the YOMConstants class
	 *
	 * @param  colorString                   Description of the Parameter
	 * @return                               The colorValue value
	 * @exception  IllegalArgumentException  Description of the Exception
	 */
	public Color getColorValue(String colorString) throws IllegalArgumentException {
		Color rv  = null;
		if (colorString != null) {
			if (colorString.startsWith("#")) {
				if (colorString.length() == 7) {
					int r  = -1;
					int g  = -1;
					int b  = -1;
					try {
						r = Integer.parseInt(colorString.substring(1, 3), 16);
						g = Integer.parseInt(colorString.substring(3, 5), 16);
						b = Integer.parseInt(colorString.substring(5, 7), 16);
					} catch (NumberFormatException nfe) {
						throw new IllegalArgumentException("String not in valid color format #XXXXXX, where X in {0,1,2,3,4,5,6,7,8,9,a,b,c,d,e,f}");
					}
					rv = new Color(r, g, b);
				} else {
					throw new IllegalArgumentException("String not in valid color format #XXXXXX, where X in {0,1,2,3,4,5,6,7,8,9,a,b,c,d,e,f}");
				}
			} else {
				try {
					rv = (Color)Color.class.getField(colorString).get(null);
				} catch (Exception e) {
					throw new IllegalArgumentException("Unknown color: " + colorString);
				}
			}
		}
		return rv;
	}


	/**
	 *  Gets the constants attribute of the YOMConstants class
	 *
	 * @return    The constants value
	 */
	public Map getConstants() {
		return constants;
	}


	/**
	 * @param  key
	 * @return
	 */
	public int getInt(String key) {
		int rv  = 0;
		try {
			rv = Integer.parseInt(getString(key));
		} catch (NumberFormatException nfe) {
			throw new NoSuchElementException(key);
		}
		return rv;
	}


	/**
	 * @param  key
	 * @return
	 */
	public String getString(String key) {
		if (!containsKey(key)) {
			return null;
		}
		String value  = constants.get(key).toString();
		return parseValue(value);
	}


	/**
	 * @param  value
	 * @return
	 */
	private String parseValue(String value) {
		Matcher m        = pattern.matcher(value);
		StringBuffer sb  = new StringBuffer();
		while (m.find()) {
			String subKey  = m.group(1);
			m.appendReplacement(sb, getString(subKey));
		}
		m.appendTail(sb);
		return sb.toString();
	}


	/**
	 * @param  newConstants  Description of the Parameter
	 */
	public void putConstants(Map newConstants) {
		constants.putAll(newConstants);
	}
}

