/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package seco.util.rdf.path;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.util.*;
import seco.util.iterator.*;
import seco.util.rdf.path.rgx.*;

/**
 *  Description of the Class
 *
 * @author     Samppa
 * @created    September 25, 2003
 */
public class RDFWalker {
	/**
	 *  Description of the Field
	 */
	private Model model;


	/**
	 *  Constructor for the RDFWalker object
	 *
	 * @param  model  Description of the Parameter
	 */
	public RDFWalker(Model model) {
		this.model = model;
	}


	/**
	 *  Gets the nsPrefixMap attribute of the RDFWalker object
	 *
	 * @return    The nsPrefixMap value
	 */
	public Map getNsPrefixMap() {
		return this.model.getGraph().getPrefixMapping().getNsPrefixMap();
	}


	/**
	 *  Gets the value attribute of the RDFWalker object
	 *
	 * @param  subject  Description of the Parameter
	 * @param  path     Description of the Parameter
	 * @return          The value value
	 */
	public BasicIterator listValues(Resource subject, RDFPath path) {
		return listValues(subject, path.getPath(), 0);
	}


	/**
	 *  Gets the value attribute of the RDFWalker object
	 *
	 * @param  subject  Description of the Parameter
	 * @param  path     Description of the Parameter
	 * @param  pos      Description of the Parameter
	 * @return          The value value
	 */
	protected BasicIterator listValues(RDFNode subject, List path, int pos) {
		BasicIterator rv  = null;
		if (pos < path.size()) {
			Link link               = (Link)path.get(pos);
			LinkedList nodes        = new LinkedList();
			BasicIterator nodeiter  = listValues(subject, link);
			if (nodeiter != null) {
				while (nodeiter.hasNext()) {
					nodes.add((RDFNode)nodeiter.next());
				}

				if (pos + 1 < path.size()) {// next link
					for (Iterator iter = nodes.iterator(); iter.hasNext(); ) {
						BasicIterator valuesiter  = listValues((RDFNode)iter.next(), path, pos + 1);
						if (valuesiter != null) {
							if (rv == null) {
								rv = valuesiter;
							} else {
								rv = rv.andThen(valuesiter);
							}
						}
					}
				} else {// nodes are the requested values
					rv = new BasicIterator(nodes.iterator());
				}
				if (link.isTransitive()) {// recurse
					for (Iterator transiter = nodes.iterator(); transiter.hasNext(); ) {
						if (rv == null) {
							rv = listValues((RDFNode)transiter.next(), path, pos);
						} else {
							rv = rv.andThen(listValues((RDFNode)transiter.next(), path, pos));
						}
					}
				}
			}
		} else {
			rv = new BasicIterator(subject);
		}
		return rv;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  subject  Description of the Parameter
	 * @param  link     Description of the Parameter
	 * @return          Description of the Return Value
	 */
	public BasicIterator listValues(RDFNode subject, Link link) {
		BasicIterator rv    = null;
		Property predicate  = this.model.getProperty(link.getProperty());
		if (predicate != null) {
			if (link.isInv()) {
				rv = new BasicIterator(this.model.listStatements(null, predicate, subject),
					new IterationMap() {
						public Object map(Object o) {
							return ((Statement)o).getSubject();
						}
					});
			} else {
				rv = new BasicIterator(this.model.listStatements((Resource)subject, predicate, (RDFNode)null),
					new IterationMap() {
						public Object map(Object o) {
							return ((Statement)o).getObject();
						}
					});
			}
		}
		return rv;
	}


	/**
	 *  The main program for the RDFWalker class
	 *
	 * @param  args  The command line arguments
	 */
	public static void main(String[] args) {
		String ERROR_MSG  =
				"Usage: java rdfpath.RDFWalker <input> <subject> <path>";

		if (args.length == 0) {
			System.out.println(ERROR_MSG);
		} else {
			try {
				Model model           = ModelFactory.createDefaultModel()
						.read(new java.io.FileInputStream(args[0]), "http://www.cs.helsinki.fi/group/seco/rdfpath#");
				RDFWalker walker      = new RDFWalker(model);
				RDFPath path          = new RGXPath(walker.getNsPrefixMap(), args[2]);
				BasicIterator values  = walker.listValues(model.getResource(args[1]), path);
				if (values != null) {
					while (values.hasNext()) {
						System.out.println(values.next());
					}
				}
			} catch (Exception e) {
				System.out.println(ERROR_MSG);
				e.printStackTrace(System.err);
			}
		}
	}
}

