/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package seco.util.rdf.path.rgx;

import java.util.*;
import java.util.regex.*;
import seco.util.rdf.path.*;
/**
 *  Description of the Class
 *
 * @author     Samppa
 * @created    September 25, 2003
 */
public class RGXPath implements RDFPath {

	/**
	 *  Description of the Field
	 */
	protected final String RGX       = "\\s*([\\^\\.]?)(\\w*):(\\w+)(\\*?)|\\s*([\\^\\.]?)\\<(.+)\\>(\\*?)";

	/**
	 *  Description of the Field
	 */
	protected ArrayList path;
	/**
	 *  Description of the Field
	 */
	protected final Pattern pattern  = Pattern.compile(RGX);


	/**
	 *  Constructor for the RGXPath object
	 *
	 * @param  nsPrefixMap           Description of the Parameter
	 * @param  pathstr               Description of the Parameter
	 * @exception  RGXPathException  Description of the Exception
	 */
	public RGXPath(Map nsPrefixMap, String pathstr) throws RGXPathException {
		LinkedList tmp   = new LinkedList();
		Matcher matcher  = pattern.matcher(pathstr);
		while (matcher.find()) {
			String property     = matcher.group(3);
			boolean inv         = false;
			boolean transitive  = false;
			//System.out.println("groups: " + matcher.groupCount());
			//for (int i = 0; i <= matcher.groupCount(); i++) {
			//	System.out.println("group " + i + "=\"" + matcher.group(i) + "\"");
			//}
			if (property != null) {
				// name space and locaname given
				String ns      = matcher.group(2);

				// Inverse?
				String tmpstr  = matcher.group(1);
				if (tmpstr != null && tmpstr.equals("^")) {
					inv = true;
				}

				// Transitive?
				tmpstr = matcher.group(4);
				if (tmpstr != null && tmpstr.equals("*")) {
					transitive = true;
				}
				if (ns != null && !ns.equals("")) {
					// name space defined?
					String nsuri  = (String)nsPrefixMap.get(ns);
					if (nsuri == null) {
						throw new RGXPathException("Unknown Name Space prefix: " + ns);
					}
					property = nsuri + property;
				} else {
					// no name space - use default
					property = "#" + property;
				}
			} else {
				// uri given
				property = matcher.group(6);
				if (property != null) {
					// Inverse?
					String tmpstr  = matcher.group(5);
					if (tmpstr != null && tmpstr.equals("^")) {
						inv = true;
					}
					// Transitive?
					tmpstr = matcher.group(7);
					if (tmpstr != null && tmpstr.equals("*")) {
						transitive = true;
					}
				} else {
					throw new RGXPathException("Malformed path expression: " + pathstr);
				}
			}
			tmp.add(new LinkImpl(property, inv, transitive));
		}
		this.path = new ArrayList(tmp);
	}


	/**
	 *  Gets the path attribute of the RGXPath object
	 *
	 * @return    The path value
	 */
	public List getPath() {
		return this.path;
	}


	/**
	 *  The main program for the RGXPath class
	 *
	 * @param  args  The command line arguments
	 */
	public static void main(String[] args) {
		try {
			HashMap nsmap    = new HashMap();
			for (int i = 0; i < args.length - 1; i++) {
				int index  = args[i].indexOf("=");
				nsmap.put(args[i].substring(0, index), args[i].substring(index + 1, args[i].length()));
			}

			RGXPath rgxpath  = new RGXPath(nsmap, args[args.length - 1]);
			List plist       = rgxpath.getPath();
			for (int i = 0; i < plist.size(); i++) {
				System.out.println(plist.get(i));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 *  Description of the Class
	 *
	 * @author     Samppa
	 * @created    September 25, 2003
	 */
	private class LinkImpl implements Link {
		/**
		 *  Description of the Field
		 */
		private boolean inv;
		/**
		 *  Description of the Field
		 */
		private String property;
		/**
		 *  Description of the Field
		 */
		private boolean transitive;


		/**
		 *  Constructor for the LinkImpl object
		 *
		 * @param  property    Description of the Parameter
		 * @param  inv         Description of the Parameter
		 * @param  transitive  Description of the Parameter
		 */
		private LinkImpl(String property, boolean inv, boolean transitive) {
			this.property = property;
			this.inv = inv;
			this.transitive = transitive;
		}


		/**
		 *  Gets the property attribute of the LinkImpl object
		 *
		 * @return    The property value
		 */
		public String getProperty() {
			return this.property;
		}


		/**
		 *  Gets the inv attribute of the LinkImpl object
		 *
		 * @return    The inv value
		 */
		public boolean isInv() {
			return inv;
		}


		/**
		 *  Gets the transitive attribute of the LinkImpl object
		 *
		 * @return    The transitive value
		 */
		public boolean isTransitive() {
			return this.transitive;
		}


		/**
		 *  Description of the Method
		 *
		 * @return    Description of the Return Value
		 */
		public String toString() {
			return (inv ? "^" + this.property : "." + this.property)
					 + (transitive ? "*" : "");
		}
	}
}


